/*
** File: mapio.c  
** Project: VxWorks Driver Framework
** Purpose: Mechanism by which OS-independent driver code can map device memory
**          into the driver's address space.
**
** (C) Copyright Alpha Data 2010
*/

#include "df.h"
#include "dfvxworks.h"

#if defined(ADB3_VXBUS3) || defined(ADB3_VXBUS4)

/* VxBus version */
void*
dfIoSpaceMap(
  DfDeviceObject* pDevObj,
  DfBusResources* pResources,
  unsigned int barIndex,
  uint64_t offset,
  uint64_t length,
  DfMemoryHandle* pHandle)
{
  VXB_DEVICE_ID pVxbDev = pDevObj->system.pVxbDev;
  DfMemoryHandle handle;
  unsigned int n = barIndex;
  int i;

  /* Find the requested memory BAR's index in the VxBus device structure; may not be the same as 'barIndex' */
  for (i = 0; i < VXB_MAXBARS; i++) {
    if (pVxbDev->regBaseFlags[i] == VXB_REG_MEM) {
      if (0 == n) {
        break;
      }
      n--;
    }
  }
  if (OK == vxbRegMap(pVxbDev, (int)i, &handle)) {
    *pHandle = handle;
    return pVxbDev->pRegBase[i];
  } else {
    return NULL;
  }
}

#else

/* Legacy version */
void*
dfIoSpaceMap(
  DfDeviceObject* pDevObj,
  DfBusResources* pResources,
  unsigned int barIndex,
  uint64_t offset,
  uint64_t length,
  DfMemoryHandle* pHandle)
{
  void* pKernelAddress = NULL;
  DfPciBar* pBar;

  /* Sanity checks */
  dfAssert(length < (unsigned long)-1);
  dfAssert(offset + length >= offset);
  dfAssert(pDevObj->hardwareId.busType == DfBusPci);
  dfAssert(barIndex < pResources->variant.pci.numMemBar);
  pBar = &pResources->variant.pci.memBar[barIndex];
  dfAssert(offset < pBar->size);
  dfAssert(offset + length <= pBar->size);

  if (!dfVxwLegacyBusToCpuAddress(pDevObj, pBar->base + offset, &pKernelAddress)) {
    return NULL;
  }

  *pHandle = NULL; /* handle is always NULL in legacy driver */
  return pKernelAddress;
}

#endif

boolean_t
dfIoSpaceUnmap(
  DfDeviceObject* pDevObj,
  void* pKernelAddress)
{
  dfAssert(NULL != pKernelAddress);

  /* Nothing to do */

  return TRUE;
}

